%%
%% Ridici proces pro databazi prenesenych mobilnich cisel
%% soubor: npdb_mob.erl
%%
-module(npdb_mob).
-behaviour(gen_server).

% O & M funlce + uzivatelske API
-export([start_link/1, stop/0, update/2, delete/1, lookup/1, fill_random/1]).

% callbacky pro gen_server
-export([init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2, code_change/3]).

%%
%% O & M funkce
%% funkce pro vypinani a zapinani procesu
%%

start_link(FileName) ->
  gen_server:start_link({local, ?MODULE}, ?MODULE, [FileName], []).

stop() ->
  gen_server:cast (?MODULE, stop).

%%
%% uzivatelelske API
%%

% vyhledavaci funkce kouka rovnou do tabulek
lookup (Msisdn) ->
  npdb_mob_db:lookup(Msisdn).

% updatovaci funkce zasilaji zpravu procesu a cekaji na odpoved
update(Msisdn, OperId) ->
  gen_server:call (?MODULE, {update, Msisdn, OperId}).

delete(Msisdn) ->
  gen_server:call (?MODULE, {delete, Msisdn}).

fill_random (N) ->
  % je treba trochu delsi timeout nez defaultnich 5 sekund
  gen_server:call (?MODULE, {fill_random, N}, 25000).

%%
%% gen_server callbacky
%%

% inicializace - vytvoreni/otevreni tabulek a nacteni dat z disku do pameti
init ([FileName]) ->
  npdb_mob_db:create_tables(FileName),
  npdb_mob_db:restore_backup(),
  State = undefined,  % stavova data nejsou potreba, mohly by zde byt nejake
                      % statistiky
  {ok, State}.

% pri uklizeni ze zavrou tabulky
terminate (_Reason, _State) ->
  npdb_mob_db:close_tables().

% zastaveni procesu
handle_cast (stop, State) ->
  {stop, normal, State}.

% obsluha synchronnich volani
% shodou okolnosti se vsude jedna jen o prenos parametru do
% prislusnych funkci pro praci s tabulkou
handle_call ({update, Msisdn, OperId}, _From, State) ->
  Reply = npdb_mob_db:update (Msisdn, OperId),
  {reply, Reply, State};

handle_call ({delete, Msisdn}, _From, State) ->
  Reply = npdb_mob_db:delete (Msisdn),
  {reply, Reply, State};

handle_call ({fill_random, N}, _From, State) ->
  Reply = npdb_mob_db:fill_random (N),
  {reply, Reply, State}.

% reakce na zpravu "mimo protokol" ve fronte
handle_info (Msg, State) ->
  io:format ("unexpected message: ~w~n", [Msg]),
  {noreply, State}.

code_change(_OldVsn, State, _Extra) ->
  {ok, State}.
