%%
%% funkce na obsluhu tabulek databaze prenesenych cisel
%% soubor: npdb_mob_db.erl
%%

-module (npdb_mob_db).
-export([create_tables/1, close_tables/0, update/2, delete/1, lookup/1, restore_backup/0]).
-export([fill_random/1, test_lookup/1, test_lookup/2, run_test_lookup/2]).

-include ("npdb.hrl").

% vytvoreni tabulek
create_tables (FileName) ->
  % parametr keypos urcuje na kterm miste v ukladane n-tici je klic
  % konstrukce #struktura.prvek se rozexpanduje na index prvku ve strukture,
  % coz je "nahodou" zrovna to co je potreba (v tomto pripade 1)
  ets:new(npdb_mob_ram, [named_table, set, {keypos, #np_mob.msisdn}]),
  dets:open_file(npdb_mob_disk, [{file, FileName}, {keypos, #np_mob.msisdn}]),
  ok.

% uzavreni tabulek
close_tables () ->
  ets:delete (npdb_mob_ram),
  dets:close (npdb_mob_disk).

% nastaveni operatora k cislu
% zapis se provadi jak do pameti tak na disk
update (Msisdn, OperId) ->
  NpRec = #np_mob{msisdn=Msisdn, oper_id=OperId},
  ets:insert(npdb_mob_ram, NpRec),
  dets:insert(npdb_mob_disk, NpRec).

% smazani zaznamu (cislo se vratilo k puvodnimu operatorovi a jiz neni prenesene)
% zapis se provadi jak do pameti tak na disk
delete (Msisdn) ->
  ets:delete(npdb_mob_ram, Msisdn),
  dets:delete(npdb_mob_disk, Msisdn).

% dohledani operatora k cislu - hleda se jen v pameti
lookup(Msisdn) ->
  case ets:lookup(npdb_mob_ram, Msisdn) of
    % vysledkem je pole o jednom prvku (tabulka je typu set, klic je zde maximalne jednou)
    [#np_mob{oper_id = OperId}] -> {ok, OperId};
    [] -> {error, instance} % nenaslo se
  end.

% po startu je treba nacist data z disku do pameti
restore_backup() ->
  ets:from_dets (npdb_mob_ram, npdb_mob_disk).

%
% funkce pro testovani
%

% nahodne cislo - jesli jsem se nespletl, tak nahodne cislo z mnoziny
% teoreticky moznych movilnich cisel v CR
% z 600000000 do 799999999
random_msisdn() ->
  600000000 + random:uniform(199999999).

% nahodne plneni N cisly
fill_random(N) ->
  io:format("generating ~w records~n", [N]),
  fill_random_acc (N, []).

% plneni bufferu strukturami v tomto pripade nahodnymi, ale mohlo by zde
% byt napr. parsovani nejakeho textoveho souboru
fill_random_acc (N, AccBuffer) when N > 0 ->
  Msisdn = random_msisdn(),
  OperId = 100 + random:uniform(3),
  NpRec = #np_mob{msisdn=Msisdn, oper_id=OperId},
  fill_random_acc (N-1, [NpRec | AccBuffer]);

% cteni dokonceno, smazat tabulky a naplnit daty
fill_random_acc (0, AccBuffer) ->

  io:format("cleaning ~w~n", [npdb_mob_ram]),
  ets:delete_all_objects(npdb_mob_ram),

  io:format("cleaning ~w~n", [npdb_mob_disk]),
  dets:delete_all_objects(npdb_mob_disk),

  io:format("filling ~w~n", [npdb_mob_ram]),
  ets:insert(npdb_mob_ram, AccBuffer),        % lze vlozit pole zaznamu

  io:format("filling ~w~n", [npdb_mob_disk]),
  dets:from_ets(npdb_mob_disk, npdb_mob_ram),
  ok.

% sekvencni provadeni opraci hledani s moznosti
% vlastni hledani vypnout (Mode - lookup a loop_only)
test_lookup(N) -> test_lookup(N, lookup).
test_lookup(N, Mode) ->
  {Time, _} = timer:tc(?MODULE, run_test_lookup, [N, Mode]),
  TimeSec = Time / 1000000,
  {ok, TimeSec, (TimeSec) / N}.

run_test_lookup(0, _) -> ok;
run_test_lookup(N, Mode) when N > 0 ->
  Msisdn = random_msisdn(),
  case Mode of
    lookup ->
      lookup(Msisdn),
      run_test_lookup(N-1, Mode);
    loop_only ->
      run_test_lookup(N-1, Mode)
  end.
